<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://developers.seeka.co
 * @since      1.0.0
 *
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/admin
 * @author     Seeka <platform@seeka.co>
 */
class Seeka_Converge_Admin
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	private $seeka_converge_options;
	private $plugin_slug;

	public $update_cache_key;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->update_cache_key = 'seeka_transient_update_cache' . $version;
		$this->plugin_slug = "seeka-converge";
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
	}

	// -------------- START OPTIONS -------------------

	public function add_plugin_page()
	{
		add_options_page(
			'Seeka', // page_title
			'Seeka', // menu_title
			'manage_options', // capability
			'seeka-converge', // menu_slug
			array($this, 'seeka_converge_create_admin_page') // function
		);
	}

	public function seeka_converge_create_admin_page()
	{
		$this->seeka_converge_options = get_option('seeka_converge_option_name');
		?>

		<div class="wrap">
			<h2>Seeka</h2>
			<p></p>
			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php
				settings_fields('seeka_converge_option_group');
				do_settings_sections('seeka-converge-admin');
				submit_button();
				?>
			</form>
		</div>
	<?php }

	public function on_option_updated($new_value)
	{
		// This fires when seeka settings updated in the plugin
		delete_transient('seeka_transient_head_content');
		delete_transient('seeka_transient_tag_cdn_content');
		delete_transient('seeka_transient_diagtag_content');
		delete_site_transient($this->update_cache_key);

		if (isset($new_value['converge_instance_id_3']) && $new_value['converge_instance_id_3']) {
			Seeka_Converge::get_or_set_unique_instanceid($new_value['converge_instance_id_3']);
		} else {
			Seeka_Converge::get_or_set_unique_instanceid();
		}

		return $new_value;
	}

	public function plugin_page_init()
	{
		register_setting(
			'seeka_converge_option_group', // option_group
			'seeka_converge_option_name', // option_name
			array($this, 'seeka_converge_sanitize') // sanitize_callback
		);

		add_settings_section(
			'seeka_converge_setting_section', // id
			'Seeka settings', // title
			array($this, 'seeka_converge_section_info'), // callback
			'seeka-converge-admin' // page
		);

		add_settings_field(
			'converge_org_0', // id
			'Organisation ID *', // title
			array($this, 'converge_org_0_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_key_0', // id
			'Public key *', // title
			array($this, 'converge_key_0_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_instance_id_3', // id
			'Instance ID *', // title
			array($this, 'converge_instance_id_3_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_debug_mode_enabled_2', // id
			'Debug mode', // title
			array($this, 'converge_debug_mode_enabled_2_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_wait_for_interaction', // id
			'Performance tweaks (alpha)', // title
			array($this, 'converge_wait_for_interaction_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_browser_sdk_version_4', // id
			'SDK version', // title
			array($this, 'converge_browser_sdk_version_4_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_endpoint_1', // id
			'Endpoint', // title
			array($this, 'converge_endpoint_1_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);

		add_settings_field(
			'converge_config_save_increment', // id
			'', // title
			array($this, 'converge_config_save_increment_callback'), // callback
			'seeka-converge-admin', // page
			'seeka_converge_setting_section' // section
		);
	}

	public function seeka_converge_sanitize($input)
	{
		$sanitary_values = array();

		if (isset($input['converge_key_0'])) {
			$sanitary_values['converge_key_0'] = sanitize_text_field($input['converge_key_0']);
		}

		if (isset($input['converge_org_0'])) {
			$sanitary_values['converge_org_0'] = sanitize_text_field($input['converge_org_0']);
		}

		if (isset($input['converge_endpoint_1'])) {
			$sanitary_values['converge_endpoint_1'] = sanitize_text_field($input['converge_endpoint_1']);
		}

		if (isset($input['converge_config_save_increment'])) {
			$sanitary_values['converge_config_save_increment'] = $input['converge_config_save_increment'] ? (intval($input['converge_config_save_increment']) + 1) : '0';
		} else {
			$sanitary_values['converge_config_save_increment'] = 0;
		}

		if (isset($input['converge_debug_mode_enabled_2'])) {
			$sanitary_values['converge_debug_mode_enabled_2'] = $input['converge_debug_mode_enabled_2'];
		}

		if (isset($input['converge_wait_for_interaction'])) {
			$sanitary_values['converge_wait_for_interaction'] = $input['converge_wait_for_interaction'];
		}

		if (isset($input['converge_instance_id_3'])) {
			$sanitary_values['converge_instance_id_3'] = sanitize_text_field($input['converge_instance_id_3']);
		}

		if (isset($input['converge_browser_sdk_version_4'])) {
			$sanitary_values['converge_browser_sdk_version_4'] = sanitize_text_field($input['converge_browser_sdk_version_4']);
		}

		return $sanitary_values;
	}

	public function seeka_converge_section_info()
	{
		printf('<a href="https://support.seeka.co/en/articles/8605608-connecting-wordpress-to-seeka" target="_blank">Need help?</a><br /><br />SDK version can be specified as <code>latest</code>, <code>alpha</code> or a specific version number.<br /><br />Debug mode should only be enabled while testing your site as it will temporarily decrease site performance while enabled.<br /><br />');
	}

	public function converge_key_0_callback()
	{
		printf(
			'<input class="regular-text" type="text" required name="seeka_converge_option_name[converge_key_0]" id="converge_key_0" value="%s">',
			isset($this->seeka_converge_options['converge_key_0']) ? esc_attr($this->seeka_converge_options['converge_key_0']) : ''
		);
	}

	public function converge_org_0_callback()
	{
		printf(
			'<input class="regular-text" type="text" required name="seeka_converge_option_name[converge_org_0]" id="converge_org_0" value="%s">',
			isset($this->seeka_converge_options['converge_org_0']) ? esc_attr($this->seeka_converge_options['converge_org_0']) : ''
		);
	}

	public function converge_endpoint_1_callback()
	{
		printf(
			'<input class="regular-text" type="text" name="seeka_converge_option_name[converge_endpoint_1]" id="converge_endpoint_1" value="%s">',
			isset($this->seeka_converge_options['converge_endpoint_1']) && $this->seeka_converge_options['converge_endpoint_1'] ? esc_attr($this->seeka_converge_options['converge_endpoint_1']) : ''
		);
	}

	public function converge_browser_sdk_version_4_callback()
	{
		printf(
			'<input class="regular-text" type="text" name="seeka_converge_option_name[converge_browser_sdk_version_4]" id="converge_browser_sdk_version_4" value="%s">',
			isset($this->seeka_converge_options['converge_browser_sdk_version_4']) && $this->seeka_converge_options['converge_browser_sdk_version_4'] ? esc_attr($this->seeka_converge_options['converge_browser_sdk_version_4']) : ''
		);
	}

	public function converge_config_save_increment_callback()
	{
		printf(
			'<input type="hidden" name="seeka_converge_option_name[converge_config_save_increment]" id="converge_config_save_increment" value="%s">',
			isset($this->seeka_converge_options['converge_config_save_increment']) && $this->seeka_converge_options['converge_config_save_increment'] ? esc_attr($this->seeka_converge_options['converge_config_save_increment']) : '0'
		);
	}

	public function converge_debug_mode_enabled_2_callback()
	{
		printf(
			'<input type="checkbox" name="seeka_converge_option_name[converge_debug_mode_enabled_2]" id="converge_debug_mode_enabled_2" value="converge_debug_mode_enabled_2" %s> <label for="converge_debug_mode_enabled_2">Enabled</label>',
			(isset($this->seeka_converge_options['converge_debug_mode_enabled_2']) && $this->seeka_converge_options['converge_debug_mode_enabled_2'] === 'converge_debug_mode_enabled_2') ? 'checked' : ''
		);
	}

	public function converge_wait_for_interaction_callback()
	{
		printf(
			'<input type="checkbox" name="seeka_converge_option_name[converge_wait_for_interaction]" id="converge_wait_for_interaction" value="converge_wait_for_interaction" %s> <label for="converge_wait_for_interaction">Enabled</label>',
			(isset($this->seeka_converge_options['converge_wait_for_interaction']) && $this->seeka_converge_options['converge_wait_for_interaction'] === 'converge_wait_for_interaction') ? 'checked' : ''
		);
	}

	public function converge_instance_id_3_callback()
	{
		printf(
			'<input class="regular-text" type="text" required name="seeka_converge_option_name[converge_instance_id_3]" id="converge_instance_id_3" value="%s">',
			isset($this->seeka_converge_options['converge_instance_id_3']) ? esc_attr($this->seeka_converge_options['converge_instance_id_3']) : ''
		);
	}

	// -------------- FINISH OPTIONS -------------------


	// -------------- START AUTO UPDATES ------------------- from https://rudrastyh.com/wordpress/self-hosted-plugin-update.html

	public function update_request()
	{

		$remote = get_transient($this->update_cache_key);

		if ("" === $remote || false === $remote || null === $remote) {

			$remote = wp_remote_get(
				'https://sdk.seeka.services/dist/wordpress/converge/latest/info.json',
				array(
					'timeout' => 10,
					'headers' => array(
						'Accept' => 'application/json'
					)
				)
			);

			if (
				is_wp_error($remote)
				|| 200 !== wp_remote_retrieve_response_code($remote)
				|| empty(wp_remote_retrieve_body($remote))
				|| $remote == null
				|| $remote == ""
			) {
				return false;
			}

			$body = json_decode(wp_remote_retrieve_body($remote));
			set_transient($this->update_cache_key, json_encode($body), 60 * 60 * 2); // 2 hours
			return $body;
		} else {
			return json_decode($remote);
		}
	}


	function update_info($res, $action, $args)
	{
		// do nothing if you're not getting plugin information right now
		if ('plugin_information' !== $action) {
			return $res;
		}

		// do nothing if it is not our plugin
		// echo $this->plugin_slug;
		// exit;
		if ($this->plugin_slug !== $args->slug) {
			return $res;
		}

		// get updates
		$remote = $this->update_request();

		if (!$remote) {
			return $res;
		}

		$res = new stdClass();

		$res->name = $remote->name;
		$res->slug = $remote->slug;
		$res->version = $remote->version;
		$res->tested = $remote->tested;
		$res->requires = $remote->requires;
		$res->author = $remote->author;
		$res->author_profile = $remote->author_profile;
		$res->download_link = $remote->download_url;
		$res->trunk = $remote->download_url;
		$res->requires_php = $remote->requires_php;
		$res->last_updated = $remote->last_updated;

		$res->sections = array(
			'description' => $remote->sections->description,
			'installation' => $remote->sections->installation,
			'changelog' => $remote->sections->changelog
		);

		if (!empty($remote->banners)) {
			$res->banners = array(
				'low' => $remote->banners->low,
				'high' => $remote->banners->high
			);
		}

		return $res;
	}

	public function enable_auto_updates_for_plugin($update, $item)
	{
		if ($item->slug === 'seeka-converge') {
			return true; // Enable auto-update for this plugin.
		}
		return $update; // Keep the default behavior for other plugins.
	}

	public function update_exec($transient)
	{
		// https://make.wordpress.org/core/2020/07/30/recommended-usage-of-the-updates-api-to-support-the-auto-updates-ui-for-plugins-and-themes-in-wordpress-5-5/

		if (empty($transient->checked)) {
			return $transient;
		}

		$remote = $this->update_request();

		if (
			$remote
			&& version_compare($this->version, $remote->version, '<')
			&& version_compare($remote->requires, get_bloginfo('version'), '<=')
			&& version_compare($remote->requires_php, PHP_VERSION, '<')
		) {
			// Update available
			$res = new stdClass();
			$res->slug = $this->plugin_slug;
			$res->plugin = 'seeka-converge/seeka-converge.php';
			$res->new_version = $remote->version;
			$res->tested = $remote->tested;
			$res->package = $remote->download_url;
			$transient->response['seeka-converge/seeka-converge.php'] = $res;
			$transient->no_update['seeka-converge/seeka-converge.php'] = null;
		} else {
			// No update available
			$item = (object) array(
				'id' => 'seeka-converge/seeka-converge.php',
				'slug' => $this->plugin_slug,
				'plugin' => 'seeka-converge/seeka-converge.php',
				'new_version' => $this->version,
				'url' => '',
				'package' => '',
				'icons' => array(),
				'banners' => array(),
				'banners_rtl' => array(),
				'tested' => '',
				'requires_php' => '',
				'compatibility' => new stdClass(),
			);
			// Adding the "mock" item to the `no_update` property is required
			// for the enable/disable auto-updates links to correctly appear in UI.
			$transient->no_update['seeka-converge/seeka-converge.php'] = $item;
			// $transient->checked['seeka-converge'] = $this->version;
		}

		return $transient;
	}

	// -------------- FINISH AUTO UPDATES -------------------


}
