<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://developers.seeka.co
 * @since      1.0.0
 *
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/includes
 * @author     Seeka <platform@seeka.co>
 */
class Seeka_Converge
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Seeka_Converge_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if (defined('Seeka_CONVERGE_VERSION')) {
			$this->version = Seeka_CONVERGE_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'seeka-converge';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Seeka_Converge_Loader. Orchestrates the hooks of the plugin.
	 * - Seeka_Converge_i18n. Defines internationalization functionality.
	 * - Seeka_Converge_Admin. Defines all hooks for the admin area.
	 * - Seeka_Converge_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-seeka-converge-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-seeka-converge-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-seeka-converge-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-seeka-converge-public.php';

		$this->loader = new Seeka_Converge_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Seeka_Converge_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$plugin_i18n = new Seeka_Converge_i18n();

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{

		$plugin_admin = new Seeka_Converge_Admin($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');


		$this->loader->add_action('admin_menu', $plugin_admin, 'add_plugin_page');
		$this->loader->add_action('admin_init', $plugin_admin, 'plugin_page_init');
		$this->loader->add_action('pre_update_option_seeka_converge_option_name', $plugin_admin, 'on_option_updated');

		$this->loader->add_filter('plugins_api', $plugin_admin, 'update_info', 20, 3);

		// Enable auto-updates for your plugin.
		$this->loader->add_filter('auto_update_plugin', $plugin_admin, 'enable_auto_updates_for_plugin', 10, 2);

		//$this->loader->add_filter('pre_set_site_transient_update_plugins', $plugin_admin, 'update_exec');
		$this->loader->add_filter('site_transient_update_plugins', $plugin_admin, 'update_exec');
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Seeka_Converge_Public($this->get_plugin_name(), $this->get_version());

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
		$this->loader->add_action('wp_head', $plugin_public, 'head');

		$this->loader->add_action('rest_api_init', $plugin_public, 'register_rest_route_seeka_tag');

		// /p prefix is for Seeka CDN / router proxy
		// add_action('rest_api_init', function () {
		// 	register_rest_route('seeka-converge/v1', '/p/static/c/(?P<unique_id>[a-zA-Z0-9-_]+)', array(
		// 		'methods' => 'GET',
		// 		'callback' => 'js_tag_api_endpoint',
		// 		'permission_callback' => 'seeka_tag_permission_callback',
		// 		'show_in_index' => true
		// 	));
		// });
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Seeka_Converge_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}

	public static function get_or_set_unique_instanceid($overwrite = null)
	{
		$seeka_plugin_instance_id = get_option('seeka_plugin_instance_id');
		if (!$seeka_plugin_instance_id) {
			$newId = $overwrite ? $overwrite : wp_generate_uuid4();
			add_option('seeka_plugin_instance_id', $newId);

			return $newId;
		} else {
			if ($overwrite) {
				update_option('seeka_plugin_instance_id', $overwrite);
				return $overwrite;
			}

			return $seeka_plugin_instance_id;
		}
	}
}
