<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://developers.seeka.co
 * @since      1.0.0
 *
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Seeka_Converge
 * @subpackage Seeka_Converge/public
 * @author     Seeka <platform@seeka.co>
 */
class Seeka_Converge_Public
{

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->plugin_name = $plugin_name;
		$this->version = $version;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Seeka_Converge_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Seeka_Converge_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/seeka-converge-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
	}

	/**
	 * Register the head for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function head()
	{
		// Use transients to store the head
		$isSidekickActive = isset($queryParams['_s_sidekick']) && $queryParams['_s_sidekick'] === '1';
		$noCache = $isSidekickActive || (isset($_GET["test"]) && $_GET["test"] == "1") || (isset($_GET["cache"]) && $_GET["cache"] == "0");

		$headContent = $noCache ? null : get_transient('seeka_transient_head_content');
		if (!$headContent) {
			$headContent = $this->generate_head();
			set_transient('seeka_transient_head_content', $headContent, 60 * 15); // 15 minutes
		}

		echo $headContent;
	}

	private function generate_head()
	{
		// Load plugin functions if not already available - bedrock compatibility
		if (!function_exists('get_plugin_data') || !function_exists('is_plugin_active')) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$seeka_converge_options = get_option('seeka_converge_option_name'); // Array of All Options
		if (!$seeka_converge_options)
			return;

		$content = "";

		# diagnosis tag		
		$content = $content . "\n" . $this->diag_script_head($seeka_converge_options);

		# script tag		
		$content = $content . "\n" . $this->script_head($seeka_converge_options);

		return $content;
	}

	private function script_head($seeka_converge_options)
	{
		$sdkVersion = array_key_exists('converge_browser_sdk_version_4', $seeka_converge_options) && $seeka_converge_options['converge_browser_sdk_version_4'] ? $seeka_converge_options['converge_browser_sdk_version_4'] : 'latest';
		$waitForInteractionStr = array_key_exists('converge_wait_for_interaction', $seeka_converge_options) && $seeka_converge_options['converge_wait_for_interaction'] ? $seeka_converge_options['converge_wait_for_interaction'] : '';
		$waitForInteraction = $waitForInteractionStr === 'converge_wait_for_interaction';
		$diag = array_key_exists('converge_debug_mode_enabled_2', $seeka_converge_options) && $seeka_converge_options['converge_debug_mode_enabled_2'] ? $seeka_converge_options['converge_debug_mode_enabled_2'] : '';
		$convergeKey = array_key_exists('converge_key_0', $seeka_converge_options) ? $seeka_converge_options['converge_key_0'] : '';
		$org = array_key_exists('converge_org_0', $seeka_converge_options) ? $seeka_converge_options['converge_org_0'] : '';
		$extraParams = "&vo=$sdkVersion";

		$convergeClientType = "wordpress";
		$convergeClientVersion = "0.0.0";
		$convergeWordpressPluginInfo = get_plugin_data(ABSPATH . 'wp-content/plugins/seeka-converge/seeka-converge.php', false, false);
		$convergeWordpressPluginVersion = '0.0.0';
		if ($convergeWordpressPluginInfo) {
			$convergeClientVersion = $convergeWordpressPluginInfo['Version'];
			$convergeWordpressPluginVersion = $convergeWordpressPluginInfo['Version'];
			$convergeClientType = "wordpress";
		}
		if (!function_exists('get_plugins') || !function_exists('is_plugin_active')) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		if (is_plugin_active('seeka-woocommerce/seeka-woocommerce.php')) {
			$convergeClientType = "wordpress/$convergeWordpressPluginVersion/woocommerce";
			$wooCommercePluginInfo = get_plugin_data(ABSPATH . 'wp-content/plugins/seeka-woocommerce/seeka-woocommerce.php', false, false);
			$convergeClientVersion = $wooCommercePluginInfo['Version'];
		}
		if ($convergeClientVersion) {
			$extraParams = $extraParams . '&client.version=' . urlencode($convergeClientVersion);
		}
		if ($convergeClientType) {
			$extraParams = $extraParams . '&client.clientType=' . urlencode($convergeClientType);
		}

		# sidekick
		$isSidekickActive = isset($_GET['_s_sidekick']) && $_GET['_s_sidekick'] === '1';
		if ($isSidekickActive) {
			$extraParams = $extraParams . '&_s_sidekick=1';
		}

		# diag
		if ($diag) {
			$extraParams = $extraParams . "&diag=$diag";
		}

		# disable cache
		$sdk_local = isset($_GET["_s_sdk_local"]) && $_GET["_s_sdk_local"] == "1";
		$disableCache = $sdk_local || (isset($_GET["_cache"]) && $_GET["_cache"] == "0") || (isset($_GET["cache"]) && $_GET["cache"] == "0");
		if ($disableCache || $isSidekickActive) {
			$dtnow = gmdate('U');
			$extraParams = $extraParams . '&cache=' . $dtnow;
		}
		$configSaveIncrement = array_key_exists('converge_config_save_increment', $seeka_converge_options) ? $seeka_converge_options['converge_config_save_increment'] : '0'; // Converge config save increment
		if ($configSaveIncrement) {
			$extraParams = $extraParams . '&csi=' . $configSaveIncrement;
		}

		// https://chatrevolution.tech/wp-json/seeka-converge/v1/p/static/c/609213cb396d9d831e9b3a0c7428ab18?k[0]=
		// http://localhost:8080/?rest_route=/seeka-converge/v1/p/static/c/ed29aeae73aaac323eca3a18e8011f3d?k[0]=

		# wordpress api url
		$restUrl = get_rest_url(null, 'seeka-converge/v1/p/static/c');
		// if (!$restUrl || $restUrl == '/index.php' || $restUrl == '/' || str_contains($restUrl, 'index.php')) {
		// if (str_contains($restUrl, 'index.php?rest_route=')) {
		// 	$restUrl = str_replace('index.php?rest_route=/', 'wp-json/', $restUrl);
		// }
		// if (!$restUrl || $restUrl == '/index.php' || $restUrl == '/') {
		// 	$restUrl = '/seeka-converge/v1/p/static/c';
		// }
		if (str_contains($restUrl, 'rest_route')) {
			// handle wordpress sites with pretty permalinks not enabled
			$extraParams = explode('?', $restUrl)[1] . '&' . $extraParams;
		}
		// echo $restUrl;
		// exit;
		$restUrlParsed = parse_url($restUrl);
		$convergeEndpoint = array_key_exists('converge_endpoint_1', $seeka_converge_options) ? $seeka_converge_options['converge_endpoint_1'] : '';
		// $defaultHost = '';

		$customSdkHost = isset($_GET['sdkh']) && $_GET['sdkh'] ? $_GET['sdkh'] : null;
		if ($customSdkHost) {
			$convergeEndpoint = $customSdkHost;
		}
		if ($convergeEndpoint && $convergeEndpoint != 'https://sdk.seeka.services') {
			try {
				// $parsedConvergeEndpoint = parse_url($convergeEndpoint);
				// $defaultHost = $parsedConvergeEndpoint['host'];
				$extraParams = $extraParams . '&sdkh=' . urlencode($convergeEndpoint);
			} catch (Exception $e) {
				error_log("Seeka - Failed to parse endpoint: $convergeEndpoint");
			}
		}

		# combine script and params
		$hostPort = isset($restUrlParsed['port']) && $restUrlParsed['port'] ? ':' . $restUrlParsed['port'] : '';
		$scriptBody = '"use strict";(()=>{var A=Object.defineProperty;var k=(i,e,t)=>e in i?A(i,e,{enumerable:!0,configurable:!0,writable:!0,value:t}):i[e]=t;var u=(i,e,t)=>k(i,typeof e!="symbol"?e+"":e,t);var l=(i,e,t)=>new Promise((n,o)=>{var a=s=>{try{r(t.next(s))}catch(d){o(d)}},c=s=>{try{r(t.throw(s))}catch(d){o(d)}},r=s=>s.done?n(s.value):Promise.resolve(s.value).then(a,c);r((t=t.apply(i,e)).next())});var g=class{constructor(e){u(this,"p");u(this,"personId");u(this,"device",{});this.p=e}getTrait(e){throw new Error("Unavailable.")}addTrait(e,t,n){return l(this,null,function*(){this.p("addTrait",[e,t,n])})}setTrait(e,t,n){return l(this,null,function*(){this.p("setTrait",[e,t,n])})}getProfile(){throw new Error("Unavailable.")}setProfile(e){return l(this,null,function*(){this.p("setProfile",[e])})}touchAndGetSessionId(){throw new Error("Unavailable.")}mergeProfile(e){return l(this,null,function*(){this.p("mergeProfile",[e])})}mergeTwoProfiles(e,t){throw new Error("Unavailable.")}reset(e){return l(this,null,function*(){this.p("reset",[e])})}syncIdentity(e){throw new Error("Unavailable.")}isAnonymous(e){throw new Error("Unavailable.")}flushIdentitySyncQueue(){return l(this,null,function*(){this.p("flushIdentitySyncQueue",[])})}sanitiseProfile(e){throw new Error("Unavailable.")}generateNewSessionId(e){throw new Error("Unavailable.")}};var f=class{constructor(e){u(this,"p");this.p=e}custom(e,t,n){this.p("custom",[e,t,n])}viewPage(e,t){this.p("viewPage",[e,t])}addPaymentMethod(e,t,n){this.p("addPaymentMethod",[e,t,n])}keywordSearch(e,t,n){this.p("keywordSearch",[e,t,n])}userLogin(e,t,n){this.p("userLogin",[e,t,n])}userLoginSignup(e,t,n){this.p("userLoginSignup",[e,t,n])}newsletterSignup(e,t,n){this.p("newsletterSignup",[e,t,n])}applyPromotionalCode(e,t,n){this.p("applyPromotionalCode",[e,t,n])}lead(e,t,n){this.p("lead",[e,t,n])}startTrial(e,t,n){this.p("startTrial",[e,t,n])}contactMessage(e,t,n){this.p("contactMessage",[e,t,n])}schedule(e,t,n){this.p("schedule",[e,t,n])}viewContentItem(e,t,n){this.p("viewContentItem",[e,t,n])}order(e,t,n){this.p("order",[e,t,n])}subscriptionItemPurchase(e,t,n){this.p("subscriptionItemPurchase",[e,t,n])}initiateCheckout(e,t,n){this.p("initiateCheckout",[e,t,n])}viewProduct(e,t,n){this.p("viewProduct",[e,t,n])}addToCart(e,t,n){this.p("addToCart",[e,t,n])}addToWishlist(e,t,n){this.p("addToWishlist",[e,t,n])}removeFromCart(e,t,n){this.p("removeFromCart",[e,t,n])}changeProductAttribute(e,t,n){this.p("changeProductAttribute",[e,t,n])}filterItemsByAttribute(e,t,n){this.p("filterItemsByAttribute",[e,t,n])}activity(e,t,n){this.p("activity",[e,t,n])}};var m=(i,e)=>{window.Converge||(window.Converge={}),window.ConvergeTrackingQueue||(window.ConvergeTrackingQueue=[]),window.Converge.track||(window.Converge.track=new f(i)),window.Converge.identity||(window.Converge.identity=new g(e)),window._skInitH||(window._skInitH=[])},v=()=>{window.addEventListener("message",i=>{let e=i.data;e.dest==="seeka"&&window.Converge[e.payload.api][e.payload.method](...e.payload.params)},!1)};var y=(i,e,t)=>{var o;if(!e||e.length===0)return t;let n=i.toLowerCase().replace("www.","").split(":")[0];for(let a=0;a<e.length;a++){let c=e[a],r=(o=c.hostname)==null?void 0:o.toLowerCase().replace("www.","").split(":")[0];if(n.endsWith(r)||r.endsWith(n))return c;let s=r.split(".").slice(1).join(".");if(n.endsWith(s)||s.endsWith(n))return c}return t};var C=i=>{let e=()=>{i.forEach(t=>{t.id&&document.getElementById(t.id)||document.body.appendChild(t)}),window.removeEventListener("scroll",e),window.removeEventListener("click",e),window.removeEventListener("keydown",e)};window.addEventListener("scroll",e),window.addEventListener("click",e),window.addEventListener("keydown",e)},p=(i,e,t,n)=>{var c,r,s;if(window._skInitH||(window._skInitH=[]),window._skInitH.indexOf(i)>-1)return;window._skInitH.push(i);let o=document.createElement("script");o.async=!0,e&&(o.onload=function(d){e(d)}),t&&(o.onerror=function(d){t(d)});let a=null;if(i.indexOf("cache=")>-1||((c=window.location.search)==null?void 0:c.indexOf("cache=0"))>-1||((r=window.location.search)==null?void 0:r.indexOf("_cache=0"))>-1||((s=window.location.search)==null?void 0:s.indexOf("_s_sidekick_mode"))>-1)a=new Date().getTime().toString();else try{a=sessionStorage.getItem("sk_config_cache")}catch(d){a=""}a&&(i=i+"&cache="+a),o.src=i,window.Seeka=window.Seeka||{},window.Seeka.client=window.Seeka.client||{};try{window.Seeka.client.origin={h:new URL(i).hostname}}catch(d){console.log(d)}if(n)C([o]);else{let d=document.getElementsByTagName("script")[0];d&&d.parentNode?d.parentNode.insertBefore(o,d):document.head.appendChild(o)}},h=i=>{if(!i||!i.instances||i.instances.length===0)return;let e={hostname:"sdk.seeka.services"};(!i.hosts||i.hosts.length===0)&&(i.hosts=[e]);let t=!1;try{t=!!sessionStorage.getItem("sk_ses_h_def")}catch(r){console.error(r)}let n=t?e:y(window.location.hostname,i.hosts,e),o=i.instances.map(r=>r.id).join("_"),a="";i.instances.forEach((r,s)=>{a+=(a?"&":"?")+`k[${s}]=${r.key}`});let c=r=>`${r.protocol||"https"}://${r.hostname}${r.path===void 0?"/static/c":r.path}/${o}${r.ext===void 0?".js":r.ext}${a}${i.extraParams?"&"+i.extraParams:""}`;p(c(n),i.onLoad,r=>{n.hostname!==e.hostname?window.location.hostname==="localhost"?console.error("Failed Seeka "+n.hostname):(console.warn("Failed Seeka "+n.hostname+". Using "+e.hostname),sessionStorage.setItem("sk_ses_h_def","1"),p(c(e),i.onLoad,()=>{console.error("Failed Seeka "+e.hostname),i.onError&&i.onError(r)})):i.onError&&i.onError(r)},i.waitForInteraction)};var I=()=>{m((i,e)=>{window.ConvergeTrackingQueue.push({t:"act",a:[i,...e]})},(i,e)=>{window.ConvergeTrackingQueue.push({t:"id",a:[i,...e]})})};I();window.SeekaInit=p;window.SeekaInstall=h;v();})();';

		$convergeInstanceId = array_key_exists('converge_instance_id_3', $seeka_converge_options) ? $seeka_converge_options['converge_instance_id_3'] : null;
		$uniqueInstanceId = Seeka_Converge::get_or_set_unique_instanceid();
		if (!$convergeInstanceId) {
			// Older installations will not have the converge instance ID baked into the site config
			// Below will just set the ID to a random ID
			// Fallback to the sdk.seeka.services CDN endpoint will fail without the user inputting the converge instance ID
			// into the wordpress Seeka plugin settings
			// Replace base64 encoded host with alphanumeric only 
			$convergeInstanceId = $uniqueInstanceId;
			$extraParams = $extraParams . '&o[0]=' . $org . '&nc=1';
		}

		// wp-json
		$installParams = array(
			'waitForInteraction' => $waitForInteraction,
			'extraParams' => $extraParams,
			'hosts' => array(
				array(
					'path' => $restUrlParsed['path'],
					'ext' => '',
					'protocol' => $restUrlParsed['scheme'],
					'hostname' => $restUrlParsed['host'] . $hostPort
				)
			),
			'instances' => array(
				array(
					'org' => $org,
					'id' => $convergeInstanceId,
					'key' => $convergeKey
				)
			)
		);

		$installScript = "SeekaInstall(" . json_encode($installParams) . ");";

		return "<script async=\"true\">$scriptBody\r\n$installScript</script>";
	}

	private function diag_script_head($seeka_converge_options)
	{
		$converge_debug_mode_enabled_2 = array_key_exists('converge_debug_mode_enabled_2', $seeka_converge_options) ? $seeka_converge_options['converge_debug_mode_enabled_2'] : false; // Converge debug mode enabled
		// $converge_config_save_increment = array_key_exists('converge_config_save_increment', $seeka_converge_options) ? $seeka_converge_options['converge_config_save_increment'] : '0'; // Converge config save increment
		if ($converge_debug_mode_enabled_2) {
			$scriptDiagTransientCacheKey = "seeka_transient_diagtag_content";
			$sdk_local = isset($_GET["_s_sdk_local"]) && $_GET["_s_sdk_local"] == "1";
			$jsContent = $sdk_local ? null : get_transient($scriptDiagTransientCacheKey);

			if (!$jsContent) {
				$converge_browser_sdk_version_4 = array_key_exists('converge_browser_sdk_version_4', $seeka_converge_options) ? $seeka_converge_options['converge_browser_sdk_version_4'] : null; // Converge browser sdk version override

				//$converge_instance_id_3 = array_key_exists('converge_instance_id_3', $seeka_converge_options) ? $seeka_converge_options['converge_instance_id_3'] : 'error'; // Converge debug log level
				$converge_key_0 = $seeka_converge_options['converge_key_0']; // Converge key
				$converge_org_0 = strtolower(str_replace('-', '', $seeka_converge_options['converge_org_0'])); // Converge org
				$convergeKeyArray = explode(',', $converge_key_0);
				$convergeOrgArray = explode(',', $converge_org_0);
				//$isMultiInstance = count($convergeKeyArray) > 1 && count($convergeOrgArray) > 1;

				$converge_endpoint_1 = array_key_exists('converge_endpoint_1', $seeka_converge_options) ? $seeka_converge_options['converge_endpoint_1'] : null; // Converge endpoint // https://localhost:22744
				if (!$converge_endpoint_1 || $converge_endpoint_1 == '' || trim($converge_endpoint_1) == '') {
					$converge_endpoint_1 = 'https://sdk.seeka.services';
				}
				// $sidekick_local = isset($_GET["_s_sidekick_local"]) && $_GET["_s_sidekick_local"] == "1";
				$sdk_local = isset($_GET["_s_sdk_local"]) && $_GET["_s_sdk_local"] == "1";

				$convergeWordpressPluginInfo = get_plugin_data(ABSPATH . 'wp-content/plugins/seeka-converge/seeka-converge.php', false, false);
				$convergeWordpressPluginVersion = null;
				$convergeClientType = "wordpress";
				$convergeClientVersion = "0.0.0";
				if ($convergeWordpressPluginInfo) {
					$convergeWordpressPluginVersion = $convergeWordpressPluginInfo['Version'];
					$convergeClientVersion = $convergeWordpressPluginVersion;
				}

				$jsUrl = "$converge_endpoint_1/static/converge/diag/tag.js?client.clientType=$convergeClientType&client.version=$convergeClientVersion";

				// Set SDK version override
				if ($converge_browser_sdk_version_4) {
					$jsUrl = $jsUrl . '&vo=' . urlencode($converge_browser_sdk_version_4);
				}

				// Append converge keys and org
				for ($i = 0; $i < count($convergeKeyArray); $i++) {
					$convergeKey = $convergeKeyArray[$i];
					$convergeOrg = $convergeOrgArray[$i];
					$jsUrl = $jsUrl . "&k[$i]=" . urlencode($convergeKey) . "&o[$i]=" . urlencode($convergeOrg);
				}

				// https localhost
				$isLocalHost = strpos($converge_endpoint_1, 'https://localhost');
				$isLocalHost = !($isLocalHost === -1 || $isLocalHost === false);

				// http localhost
				if (!$isLocalHost) {
					$isLocalHost = strpos($converge_endpoint_1, 'http://localhost');
					$isLocalHost = !($isLocalHost === -1 || $isLocalHost === false);
				}

				$response = wp_remote_get($jsUrl, array(
					'timeout' => $isLocalHost ? 240 : 10,
					'httpversion' => '1.1',
					'sslverify' => !$isLocalHost,
					'headers' => array(
						'referer' => home_url()
					)
				));

				if (!is_wp_error($response) && (200 === wp_remote_retrieve_response_code($response))) {
					$jsContent = $response['body'];
					set_transient($scriptDiagTransientCacheKey, $jsContent, 60 * 60 * 1); // 1 hour
				} else {
					return "
					<script>
						throw new Error('Seeka - Failed to load diag script - responseCode " . wp_remote_retrieve_response_code($response) . ", url " . $jsUrl . "');
					</script>";
				}
			}

			if ($jsContent) {
				return "<script>$jsContent</script>";
			}
		}
	}

	function register_rest_route_seeka_tag()
	{
		register_rest_route('seeka-converge/v1', '/p/static/c/(?P<unique_id>[a-zA-Z0-9-_]+)', array(
			'methods' => 'GET',
			'callback' => 'js_tag_api_endpoint',
			'permission_callback' => 'seeka_tag_permission_callback',
			'show_in_index' => true
		));
	}
}

function js_tag_api_endpoint($data)
{
	$id = $data->get_param('unique_id');
	$queryParams = $data->get_query_params();

	handle_tag_http($queryParams, $id);
}

function seeka_tag_permission_callback()
{
	return true;
}

function handle_tag_http($queryParams, $seekaUniqueInstanceId)
{
	$cacheControl = 'no-store, no-cache, must-revalidate';
	$contentType = 'application/javascript';
	$expires = 'Thu, 01-Jan-70 00:00:01 GMT';
	$lastModified = '';
	$md5ETag = '';
	$statusCode = 200;
	$sdkHost = 'https://sdk.seeka.services';
	if (isset($queryParams['sdkh']) && $queryParams['sdkh']) {
		$sdkHost = $queryParams['sdkh'];
	}

	// Create url
	$path = "static/c/$seekaUniqueInstanceId.js";
	if (isset($queryParams['nc']) && $queryParams['nc']) {
		// Has no converge instance ID set in Seeka plugin settings, all new installations should have converge instance id
		// Below is to cater for legacy installs
		$path = "static/converge/tag.js";
	}
	$jsUrl = "$sdkHost/$path?" . $_SERVER['QUERY_STRING'];

	$isSidekickActive = isset($queryParams['_s_sidekick']) && $queryParams['_s_sidekick'] === '1';
	$disableCache = isset($queryParams['cache']) && $queryParams['cache'];

	// https localhost
	$isLocalHost = strpos($sdkHost, 'https://localhost');
	$isLocalHost = !($isLocalHost === -1 || $isLocalHost === false);

	// http localhost
	if (!$isLocalHost) {
		$isLocalHost = strpos($sdkHost, 'http://localhost');
		$isLocalHost = !($isLocalHost === -1 || $isLocalHost === false);
	}

	$transientKey = "seeka_transient_js_tag_cdn_content_$seekaUniqueInstanceId";
	$cachedContent = $disableCache ? '' : get_transient($transientKey);
	$jsContent = $cachedContent ? $cachedContent : '';
	$enableCacheHeaders = $cachedContent ? true : false;

	if (!$jsContent) {
		try {
			$ch = curl_init();
			$headers = array(
				'Referrer' => "https://" . $_SERVER['SERVER_NAME'] . $_SERVER['REQUEST_URI']
			);
			curl_setopt($ch, CURLOPT_URL, $jsUrl);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_FAILONERROR, true);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

			if ($isLocalHost) {
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
			}
			curl_setopt($ch, CURLOPT_TIMEOUT, $isLocalHost ? 240 : 15); // Timeout in seconds

			$curlResponse = curl_exec($ch);
			$http_status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
			$curl_errno = curl_errno($ch);

			if ($curlResponse && !$curl_errno) {
				$jsContent = $curlResponse;
				$md5ETag = md5($jsContent);
				set_transient($transientKey, $jsContent, 60 * 15); // 15 min
				// set cache headers
				if (!$isLocalHost && !$isSidekickActive && !$disableCache) {
					$enableCacheHeaders = true;
				}
			} else {
				$statusCode = !$http_status || $http_status === 0 ? 503 : $http_status;
				$jsContent = 'console.error("Seeka - Failed to load base script - responseCode ' . $http_status . ', url ' . $jsUrl . '");';
			}
		} catch (Exception $ex) {
			$statusCode = 503;

			$jsContent = 'console.error("Seeka - Failed to load base script", ' . json_encode($ex->getMessage()) . ',"' . $jsUrl . '");';
		}
	}

	if ($enableCacheHeaders) {
		$cacheControl = 'private, max-age=1800'; // 15 min (60 sec * 15)					
		$lastModified = gmdate('D, d M Y H:i:s') . ' GMT';
		$expires = '';
	}

	$scriptContent = "window.Seeka = window.Seeka || {};\nwindow.Seeka.client = window.Seeka.client || {};\nwindow.Seeka.client.origin = { h: '" . $_SERVER['SERVER_NAME'] . "' };\n" . $jsContent;

	// Error response code will cause seeka to try the standard CDN endpoint to load the script
	http_response_code($statusCode);

	if ($md5ETag) {
		header('Etag: ' . $md5ETag);
	}
	if ($expires) {
		header('Expires: ' . $expires);
	}
	if ($lastModified) {
		header('Last-Modified: ' . $lastModified);
	}
	header('Cache-Control: ' . $cacheControl);
	header('Content-Type: ' . $contentType);

	echo $scriptContent;

	exit();
}
